@@ -1,9 +1,11 @@
 package SVN::Notify;
 
-# $Id: Notify.pm 2198 2005-11-10 23:15:18Z theory $
+# $Id: Notify.pm 2790 2006-04-06 22:24:58Z theory $
 
 use strict;
-$SVN::Notify::VERSION = '2.50';
+use constant WIN32  => $^O eq 'MSWin32';
+use constant PERL58 => $] > 5.007;
+$SVN::Notify::VERSION = '2.57';
 
 =begin comment
 
@@ -42,7 +44,7 @@ Use the class in a custom script:
 
 This class may be used for sending email messages for Subversion repository
 activity. There are a number of different modes supported, and SVN::Notify is
-fully subclassable, to easily add new functionality. By default, A list of all
+fully subclassable to easily add new functionality. By default, A list of all
 the files affected by the commit will be assembled and listed in a single
 message. An additional option allows diffs to be calculated for the changes
 and either appended to the message or added as an attachment. See the
@@ -60,10 +62,12 @@ everything executes properly.
 =cut
 
 # Map the svnlook changed codes to nice labels.
-my %map = ( U => 'Modified Paths',
-            A => 'Added Paths',
-            D => 'Removed Paths',
-            _ => 'Property Changed');
+my %map = (
+    U => 'Modified Paths',
+    A => 'Added Paths',
+    D => 'Removed Paths',
+    _ => 'Property Changed',
+);
 
 ##############################################################################
 
@@ -167,12 +171,16 @@ domain specified by the C<user_domain> parameter.
 =item svnlook
 
   svnnotify --svnlook /path/to/svnlook
-  svnnotify --l /path/to/svnlook
+  svnnotify -l /path/to/svnlook
 
-The location of the F<svnlook> executable. The default is
-F</usr/local/bin/svnlook>. Specify a different path or set the C<$SVNLOOK>
-environment variable if this is not the location of F<svnlook> on your
-box. It's important to provide a complete path to F<svnlook> because the
+The location of the F<svnlook> executable. If not specified, SVN::Notify will
+search through the directories in the C<$PATH> environment variable, plus in
+F</usr/local/bin> and F</usr/sbin>, for an F<svnlook> executable. Specify a
+full path to F<svnlook> via this option or by setting the C<$SVNLOOK>
+environment variable if F<svnlook> isn't in your path or to avoid loading
+L<File::Spec|File::Spec>.
+
+It's important to provide a complete path to F<svnlook> because the
 environment during the execution of F<post-commit> is anemic, with nary a
 C<$PATH> environment variable to be found. So if F<svnnotify> appears not to
 be working at all (and Subversion seems loathe to log when it dies!), make
@@ -182,13 +190,25 @@ executable.
 =item sendmail
 
   svnnotify --sendmail /path/to/sendmail
-  svnnotify --s /path/to/sendmail
+  svnnotify -s /path/to/sendmail
 
-The location of the F<sendmail> executable. The default is
-F</usr/sbin/sendmail>. Specify a different path or set the C<$SENDMAIL>
-environment variable if this is not the location of F<sendmail> on your box.
-The same caveats as applied to the location of the F<svnlook> executable
-apply here.
+The location of the F<sendmail> executable. If neither the C<sendmail> nor the
+C<smtp> parameter is specified, SVN::Notify will search through the
+directories in the C<$PATH> environment variable, plus in F</usr/local/bin>
+and F</usr/sbin>, for an F<sendmail> executable. Specify a full path to
+F<sendmail> via this option or by setting the C<$SENDMAIL> environment
+variable if F<sendmail> isn't in your path or to avoid loading
+L<File::Spec|File::Spec>. The same caveats as applied to the location of the
+F<svnlook> executable apply here.
+
+=item smtp
+
+  svnnotify --smtp smtp.example.com
+
+The address for an SMTP server through which to send the notification email.
+If unspecified, SVN::Notify will use F<sendmail> to send the message. If
+F<sendmail> is not installed locally (such as on Windows boxes!), you I<must>
+specify an SMTP server.
 
 =item charset
 
@@ -199,6 +219,19 @@ The character set typically used on the repository for log messages, file
 names, and file contents. Used to specify the character set in the email
 Content-Type headers. Defaults to "UTF-8".
 
+=item io_layer
+
+  svnnotify --io-layer raw
+  svnnotify -o bytes
+
+The Perl IO layer to use for inputting and outputting data. See
+L<perlio|perlio> for details. Defaults to "encoding($charset)". If your
+repository uses different character encodings, C<charset> should be set to
+whatever is the most common character encoding, and C<io_layer> is best set to
+C<raw>. In that case, some characters might not look right in the commit
+messaage (because an email can manage only one character encoding at a time),
+but then C<svnnotify> won't get stuck inssuing a slew of warnings.
+
 =item language
 
   svnnotify --language fr
@@ -232,7 +265,7 @@ implicitly sets the C<with_diff> parameter to a true value.
 =item reply_to
 
   svnnotify --reply-to devlist@example.com
-  svnnotify --R developers@example.net
+  svnnotify -R developers@example.net
 
 The email address to use in the "Reply-To" header of the notification email.
 No "Reply-To" header will be added to the email if no value is specified for
@@ -274,12 +307,26 @@ Pass an array reference if calling C<new()> directly.
 =item no_first_line
 
   svnnotify --no-first-line
-  svnnotify --O
+  svnnotify -O
 
 Omits the first line of the log message from the subject. This is most useful
 when used in combination with the C<subject_cx> parameter, so that just the
 commit context is displayed in the subject and no part of the log message.
 
+=item header
+
+  svnnotify --header 'SVN::Notify is brought to you by Kineticode.
+
+Adds a specified text to each message as a header at the beginning of the
+body of the message.
+
+=item footer
+
+  svnnotify --footer 'Copyright (R) by Kineticode, Inc.'
+
+Adds a specified text to each message as a footer at the end of the body of
+the message.
+
 =item max_sub_length
 
   svnnotify --max-sub-length 72
@@ -292,6 +339,16 @@ shorter. This could potentially be quite long. To prevent the subject from
 being over a certain number of characters, specify a maximum length here, and
 SVN::Notify will truncate the subject to the last word under that length.
 
+=item max_diff_length
+
+  svnnotify --max-diff-length 1024
+
+The maximum length of the diff (attached or in the body). The diff output is
+truncated at the last line under the maximum character count specified and
+then outputs an additional line indicating that the maximum diff size was
+reached and output truncated. This is helpful when a large diff output could
+cause a message to bounce due to message size.
+
 =item handler
 
   svnnotify --handler HTML
@@ -300,8 +357,10 @@ SVN::Notify will truncate the subject to the last word under that length.
 Specify the subclass of SVN::Notify to be constructed and returned, and
 therefore to handle the notification. Of course you can just use a subclass
 directly, but this parameter is designed to make it easy to just use
-C<< SVN::Notify->new >> without worrying about loading subclasses, such as
-in F<svnnotify>.
+C<< SVN::Notify->new >> without worrying about loading subclasses, such as in
+F<svnnotify>. Be sure to read the documentation for your subclass of choice,
+as there may be additional parameters and existing parameters may behave
+differently.
 
 =item svnweb_url
 
@@ -429,11 +488,13 @@ sub new {
       unless $params{to} || $params{to_regex_map};
 
     # Set up default values.
-    $params{svnlook}   ||= $ENV{SVNLOOK}  || '/usr/local/bin/svnlook';
-    $params{sendmail}  ||= $ENV{SENDMAIL} || '/usr/sbin/sendmail';
+    $params{svnlook}   ||= $ENV{SVNLOOK}  || $class->find_exe('svnlook');
     $params{with_diff} ||= $params{attach_diff};
     $params{verbose}   ||= 0;
     $params{charset}   ||= 'UTF-8';
+    $params{io_layer}  ||= "encoding($params{charset})";
+    $params{sendmail}  ||= $ENV{SENDMAIL} || $class->find_exe('sendmail')
+        unless $params{smtp};
 
     # svnweb_url and viewcvs_url are mutually exlusive.
     if ($params{svnweb_url} && $params{svnweb_url} !~ /%s/) {
@@ -529,37 +590,42 @@ sub get_options {
 
     # Get options.
     Getopt::Long::GetOptions(
-        "repos-path|p=s"      => \$opts->{repos_path},
-        "revision|r=s"        => \$opts->{revision},
-        "to|t=s"              => \$opts->{to},
-        "to-regex-map|x=s%"   => \$opts->{to_regex_map},
-        "from|f=s"            => \$opts->{from},
-        "user-domain|D=s"     => \$opts->{user_domain},
-        "svnlook|l=s"         => \$opts->{svnlook},
-        "sendmail|s=s"        => \$opts->{sendmail},
-        "charset|c=s"         => \$opts->{charset},
-        "language|g=s"        => \$opts->{language},
-        "with-diff|d"         => \$opts->{with_diff},
-        "attach-diff|a"       => \$opts->{attach_diff},
-        "reply-to|R=s"        => \$opts->{reply_to},
-        "subject-prefix|P=s"  => \$opts->{subject_prefix},
-        "subject-cx|C"        => \$opts->{subject_cx},
-        "strip-cx-regex|X=s@" => \$opts->{strip_cx_regex},
-        "no-first-line|O"     => \$opts->{no_first_line},
-        "max-sub-length|i=i"  => \$opts->{max_sub_length},
-        "handler|H=s"         => \$opts->{handler},
-        "viewcvs-url|U=s"     => \$opts->{viewcvs_url},
-        "svnweb-url|S=s"      => \$opts->{svnweb_url},
-        "rt-url|T=s"          => \$opts->{rt_url},
-        "bugzilla-url|B=s"    => \$opts->{bugzilla_url},
-        "jira-url|J=s"        => \$opts->{jira_url},
-        "gnats-url|G=s"       => \$opts->{gnats_url},
-        "ticket-url=s"        => \$opts->{ticket_url},
-        "ticket-regex=s"      => \$opts->{ticket_regex},
-        "verbose|V+"          => \$opts->{verbose},
-        "help|h"              => \$opts->{help},
-        "man|m"               => \$opts->{man},
-        "version|v"           => \$opts->{version},
+        'repos-path|p=s'      => \$opts->{repos_path},
+        'revision|r=s'        => \$opts->{revision},
+        'to|t=s'              => \$opts->{to},
+        'to-regex-map|x=s%'   => \$opts->{to_regex_map},
+        'from|f=s'            => \$opts->{from},
+        'user-domain|D=s'     => \$opts->{user_domain},
+        'svnlook|l=s'         => \$opts->{svnlook},
+        'sendmail|s=s'        => \$opts->{sendmail},
+        'smtp=s'              => \$opts->{smtp},
+        'charset|c=s'         => \$opts->{charset},
+        'io-layer|o=s'        => \$opts->{io_layer},
+        'language|g=s'        => \$opts->{language},
+        'with-diff|d'         => \$opts->{with_diff},
+        'attach-diff|a'       => \$opts->{attach_diff},
+        'reply-to|R=s'        => \$opts->{reply_to},
+        'subject-prefix|P=s'  => \$opts->{subject_prefix},
+        'subject-cx|C'        => \$opts->{subject_cx},
+        'strip-cx-regex|X=s@' => \$opts->{strip_cx_regex},
+        'no-first-line|O'     => \$opts->{no_first_line},
+        'max-sub-length|i=i'  => \$opts->{max_sub_length},
+        'max-diff-length|e=i' => \$opts->{max_diff_length},
+        'handler|H=s'         => \$opts->{handler},
+        'viewcvs-url|U=s'     => \$opts->{viewcvs_url},
+        'svnweb-url|S=s'      => \$opts->{svnweb_url},
+        'rt-url|T=s'          => \$opts->{rt_url},
+        'bugzilla-url|B=s'    => \$opts->{bugzilla_url},
+        'jira-url|J=s'        => \$opts->{jira_url},
+        'gnats-url|G=s'       => \$opts->{gnats_url},
+        'ticket-url=s'        => \$opts->{ticket_url},
+        'ticket-regex=s'      => \$opts->{ticket_regex},
+        'verbose|V+'          => \$opts->{verbose},
+        'help|h'              => \$opts->{help},
+        'man|m'               => \$opts->{man},
+        'version|v'           => \$opts->{version},
+        'header=s'            => \$opts->{header},
+        'footer=s'            => \$opts->{footer},
     ) or return;
 
     # Load a subclass if one has been specified.
@@ -599,6 +665,34 @@ sub file_label_map { \%map }
 
 ##############################################################################
 
+=head3 find_exe
+
+  my $exe = SVN::Notify->find_exe($exe_name);
+
+This method searches through the system path, as well as the extra directories
+F</usr/local/bin> and F</usr/sbin> (because they're common paths for
+C<svnlook> and C<sendmail> for an executable file with the name C<$exe_name>.
+The first one it finds is returned with its full path. If none is found,
+C<find_exe()> returns undef.
+
+=cut
+
+sub find_exe {
+    my ($class, $exe) = @_;
+    $exe .= '.exe' if WIN32;
+    require File::Spec;
+    for my $path (
+        File::Spec->path, qw(/usr/local/bin /usr/bin /usr/sbin),
+        'C:\\program files\\subversion\\bin'
+    ) {
+        my $file = File::Spec->catfile($path, $exe);
+        return $file if -f $file && -x _;
+    }
+    return;
+}
+
+##############################################################################
+
 =head1 Instance Interface
 
 =head2 Instance Methods
@@ -655,8 +749,9 @@ sub prepare_recipients {
     $self->_dbpnt( "Preparing recipients list") if $self->{verbose};
     return $self unless $self->{to_regex_map} || $self->{subject_cx};
     my @to = $self->{to} ? ($self->{to}) : ();
-    my $regexen = delete $self->{to_regex_map};
+    my $regexen = $self->{to_regex_map};
     if ($regexen) {
+        $regexen = {%$regexen};
         $self->_dbpnt( "Compiling regex_map regular expressions")
           if $self->{verbose} > 1;
         for (values %$regexen) {
@@ -669,7 +764,7 @@ sub prepare_recipients {
         $regexen = {};
     }
 
-    my ($len, $cx);
+    my $cx;
     my $fh = $self->_pipe('-|', $self->{svnlook}, 'dirs-changed',
                           $self->{repos_path}, '-r', $self->{revision});
 
@@ -688,10 +783,12 @@ sub prepare_recipients {
         if ($self->{subject_cx}) {
             # XXX Do we need to set utf8 here?
             my $l = length;
-            ($len, $cx) = ($l, $_) unless defined $len && $len < $l;
-            $self->_dbpnt( qq{Context is "$cx"}) if $self->{verbose} > 1;
+            $cx ||= $_;
+            $cx =~ s{[/\\]?[^/\\]+$}{} until !$cx || /^$cx/;
         }
     }
+    $self->_dbpnt( qq{Context is "$cx"})
+        if $self->{subject_cx} && $self->{verbose} > 1;
     close $fh or warn "Child process exited: $?\n";
     $self->{to} = join ', ', @to;
     $self->{cx} = $cx;
@@ -843,9 +940,9 @@ sub prepare_subject {
   $notifier->execute;
 
 Sends the notification message. This involves opening a file handle to
-F<sendmail> and passing it to C<output()>. This is the main method used to
-send notifications or execute any other actions in response to Subversion
-activity.
+F<sendmail> or a tied file handle connected to an SMTP server and passing it
+to C<output()>. This is the main method used to send notifications or execute
+any other actions in response to Subversion activity.
 
 =cut
 
@@ -854,8 +951,9 @@ sub execute {
     $self->_dbpnt( "Sending message") if $self->{verbose};
     return $self unless $self->{to};
 
-    # Safe pipe to sendmail. See perlipc(1).
-    my $out = $self->_pipe('|-', $self->{sendmail}, '-oi', '-t');
+    my $out = $self->{smtp}
+        ? SVN::Notify::SMTP->get_handle($self)
+        : $self->_pipe('|-', $self->{sendmail}, '-oi', '-t');
 
     # Output the message.
     $self->output($out);
@@ -870,26 +968,29 @@ sub execute {
 =head3 output
 
   $notifier->output($file_handle);
+  $notifier->output($file_handle, $no_headers);
 
 Called internally by C<execute()> to output a complete email message. The file
-handle passed is a piped connection to F<sendmail>, so that C<output()> and
-its related methods can print directly to sendmail.
+a file handle, so that C<output()> and its related methods can print directly
+to the email message. The optional second argument, if true, will suppress the
+output of the email headers.
 
 Really C<output()> is a simple wrapper around a number of other method calls.
 It is thus essentially a shortcut for:
 
-    $notifier->output_headers($out);
+    $notifier->output_headers($out) unless $no_headers;
     $notifier->output_content_type($out);
     $notifier->start_body($out);
-    $self->output_metadata($out);
+    $notifier->output_metadata($out);
     $notifier->output_log_message($out);
     $notifier->output_file_lists($out);
     if ($notifier->with_diff) {
+        my $diff_handle = $self->diff_handle;
         if ($notifier->attach_diff) {
             $notifier->end_body($out);
-            $notifier->output_attached_diff($out);
+            $notifier->output_attached_diff($out, $diff_handle);
         } else {
-            $notifier->output_diff($out);
+            $notifier->output_diff($out, $diff_handle);
             $notifier->end_body($out);
         }
     } else {
@@ -900,10 +1001,9 @@ It is thus essentially a shortcut for:
 =cut
 
 sub output {
-    my ($self, $out) = @_;
+    my ($self, $out, $no_headers) = @_;
     $self->_dbpnt( "Outputting notification message") if $self->{verbose} > 1;
-
-    $self->output_headers($out);
+    $self->output_headers($out) unless $no_headers;
     $self->output_content_type($out);
     $self->start_body($out);
     $self->output_metadata($out);
@@ -911,8 +1011,7 @@ sub output {
     $self->output_file_lists($out);
     if ($self->{with_diff}) {
         # Get a handle on the diff output.
-        my $diff = $self->_pipe('-|', $self->{svnlook}, 'diff',
-                                $self->{repos_path}, '-r', $self->{revision});
+        my $diff = $self->diff_handle;
         if ($self->{attach_diff}) {
             $self->end_body($out);
             $self->output_attached_diff($out, $diff);
@@ -934,10 +1033,8 @@ sub output {
 
   $notifier->output_headers($file_handle);
 
-Outputs the headers for the notification message. If the C<attach_diff>
-parameter was set to true, then a boundary string will be generated and the
-Content-Type set to "multipart/mixed" and stored as the C<boundary>
-attribute.
+Outputs the headers for the notification message headers. Should be called
+only once for a single email message.
 
 =cut
 
@@ -950,9 +1047,34 @@ sub output_headers {
       "To: $self->{to}\n",
       "Subject: $self->{subject}\n";
     print $out "Reply-To: $self->{reply_to}\n" if $self->{reply_to};
-    print $out "X-Mailer: SVN::Notify " . $self->VERSION .
-      ": http://search.cpan.org/dist/SVN-Notify/\n";
+    print $out "X-Mailer: SVN::Notify ", $self->VERSION,
+               ": http://search.cpan.org/dist/SVN-Notify/\n";
 
+    return $self;
+}
+
+##############################################################################
+
+=head3 output_content_type
+
+  $notifier->output_content_type($file_handle);
+
+Outputs the content type and transfer encoding headers. These demarcate the
+body of the message. If the C<attach_diff> parameter was set to true, then a
+boundary string will be generated and the Content-Type set to
+"multipart/mixed" and stored as the C<boundary> attribute.
+
+Arter that, this method outputs the content type returned by
+C<content_type()>, the character set specified by the C<charset> attribute,
+and a Content-Transfer-Encoding of "8bit". Subclasses can either rely on this
+functionality or override this method to provide their own content type
+headers.
+
+=cut
+
+sub output_content_type {
+    my ($self, $out) = @_;
+    $self->_dbpnt( "Outputting content type") if $self->{verbose} > 2;
     # Output the content type.
     if ($self->{attach_diff}) {
         # We need a boundary string.
@@ -965,27 +1087,6 @@ sub output_headers {
           qq{Content-Type: multipart/mixed; boundary="$self->{boundary}"\n\n};
     }
 
-    return $self;
-}
-
-##############################################################################
-
-=head3 output_content_type
-
-  $notifier->output_content_type($file_handle);
-
-Outputs the content type and transfer encoding headers. These demarcate the
-body of the message. This method outputs the content type returned by
-C<content_type()>, the character set specified by the C<charset> attribute,
-and a Content-Transfer-Encoding of "8bit". Subclasses can either rely on this
-functionality or override this method to provide their own content type
-headers.
-
-=cut
-
-sub output_content_type {
-    my ($self, $out) = @_;
-    $self->_dbpnt( "Outputting content type") if $self->{verbose} > 2;
     my $ctype = $self->content_type;
     print $out "--$self->{boundary}\n" if $self->{attach_diff};
     print $out "Content-Type: $ctype; charset=$self->{charset}\n",
@@ -1000,12 +1101,17 @@ sub output_content_type {
 
   $notifier->start_body($file_handle);
 
-This method starts the body of the notification message. It doesn't actually
-do anything in this class, but see subclasses for other behaviors.
+This method starts the body of the notification message, which means that it
+outputs the contents of the C<header> attribute, if there are any. Otherwise
+it outputs nothing, but see subclasses for other behaviors.
 
 =cut
 
-sub start_body { shift }
+sub start_body {
+    my ($self, $out) = @_;
+    print $out "$self->{header}\n\n" if $self->{header};
+    return $self;
+}
 
 ##############################################################################
 
@@ -1152,6 +1258,7 @@ sub output_file_lists {
         print $out "\n$map->{$type}:\n$dash{$type}\n";
         print $out "    $_\n" for @{ $files->{$type} };
     }
+    print $out "\n";
 }
 
 ##############################################################################
@@ -1160,15 +1267,17 @@ sub output_file_lists {
 
   $notifier->end_body($file_handle);
 
-Closes out the body of the email. Designed to be called when the body of the
-message is complete, and before any call to C<output_attached_diff()>.
+Closes out the body of the email by outtputing the contents of the C<footer>
+attribute, if any, and then a couple of newlines. Designed to be called when
+the body of the message is complete, and before any call to
+C<output_attached_diff()>.
 
 =cut
 
 sub end_body {
     my ($self, $out) = @_;
     $self->_dbpnt( "Ending body") if $self->{verbose} > 2;
-    print $out "\n\n";
+    print $out $self->{footer} ? "\n$self->{footer}\n" : "\n";
     return $self;
 }
 
@@ -1230,18 +1339,58 @@ sub end_message {
     return $self;
 }
 
+##############################################################################
+
+=head3 diff_handle
+
+  my $diff = $notifier->diff_handle;
+  while (<$diff>) { print }
+
+Returns a file handle reference providing access to the the commit diff. It
+will usually be passed as the second argument to C<output_diff()> or
+C<output_attached_diff()>.
+
+=cut
+
+sub diff_handle {
+    my $self = shift;
+    return $self->_pipe(
+        '-|'   => $self->{svnlook},
+        'diff' => $self->{repos_path},
+        '-r'   => $self->{revision}
+    );
+}
+
 ##############################################################################
 # This method actually dumps the output of C<svnlook diff>. It's a separate
 # method because output_attached_diff() and output_diff() do essentially the
-# same thing, so they can both call it.
+# same thing, so they can both call it. The diff output will be truncated at
+# max_diff_length, if specified.
 ##############################################################################
 
 sub _dump_diff {
     my ($self, $out, $diff) = @_;
 
-    while (<$diff>) {
-        s/[\n\r]+$//;
-        print $out "$_\n";
+    if (my $max = $self->{max_diff_length}) {
+        my $length = 0;
+        while (<$diff>) {
+            s/[\n\r]+$//;
+            if (($length += length) < $max) {
+                print $out "$_\n";
+            }
+            else {
+                print $out
+                    "\n\@\@ Diff output truncated at $max characters. \@\@\n";
+                last;
+            }
+        }
+    }
+
+    else {
+        while (<$diff>) {
+            s/[\n\r]+$//;
+            print $out "$_\n";
+        }
     }
     close $diff or warn "Child process exited: $?\n";
     return $self;
@@ -1258,7 +1407,9 @@ __PACKAGE__->_accessors(qw(
     user_domain
     svnlook
     sendmail
+    smtp
     charset
+    io_layer
     language
     with_diff
     attach_diff
@@ -1266,6 +1417,7 @@ __PACKAGE__->_accessors(qw(
     subject_prefix
     subject_cx
     max_sub_length
+    max_diff_length
     viewcvs_url
     svnweb_url
     rt_url
@@ -1274,6 +1426,8 @@ __PACKAGE__->_accessors(qw(
     gnats_url
     ticket_url
     ticket_regex
+    header
+    footer
     verbose
     boundary
     user
@@ -1362,6 +1516,13 @@ Gets or sets the value of the C<svnlook> attribute.
 
 Gets or sets the value of the C<sendmail> attribute.
 
+=head3 smtp
+
+  my $smtp = $notifier->smtp;
+  $notifier = $notifier->smtp($smtp);
+
+Gets or sets the value of the C<smtp> attribute.
+
 =head3 charset
 
   my $charset = $notifier->charset;
@@ -1369,6 +1530,13 @@ Gets or sets the value of the C<sendmail> attribute.
 
 Gets or sets the value of the C<charset> attribute.
 
+=head3 io_layer
+
+  my $io_layer = $notifier->io_layer;
+  $notifier = $notifier->io_layer($io_layer);
+
+Gets or sets the value of the C<io_layer> attribute.
+
 =head3 language
 
   my $language = $notifier->language;
@@ -1418,6 +1586,13 @@ Gets or sets the value of the C<subject_cx> attribute.
 
 Gets or sets the value of the C<max_sub_length> attribute.
 
+=head3 max_diff_length
+
+  my $max_diff_length = $notifier->max_diff_length;
+  $notifier = $notifier->max_diff_length($max_diff_length);
+
+Gets or set the value of the C<max_diff_length> attribute.
+
 =head3 svnweb_url
 
   my $svnweb_url = $notifier->svnweb_url;
@@ -1498,6 +1673,20 @@ Gets or sets the value of the C<files> attribute, which is set to a hash
 reference of change type mapped to arrays of strings by the call to
 C<prepare_files()>.
 
+=head3 header
+
+  my $header = $notifier->header;
+  $notifier = $notifier->header($header);
+
+Gets or set the value of the C<header> attribute.
+
+=head3 footer
+
+  my $footer = $notifier->footer;
+  $notifier = $notifier->footer($footer);
+
+Gets or set the value of the C<footer> attribute.
+
 =cut
 
 ##############################################################################
@@ -1513,17 +1702,22 @@ sub _pipe {
     $self->_dbpnt( "Piping execution of '" . join("', '", @_) . "'")
       if $self->{verbose};
     # Safer version of backtick (see perlipc(1)).
-    # XXX Use Win32::Process on Win32? This doesn't seem to work as-is on Win32.
     local *PIPE;
-    my $pid = open(PIPE, $mode);
+    if (WIN32) {
+        my $cmd = $mode eq '-|' ? join(q{ }, @_) . '|' : '|' . join q{ }, @_;
+        open PIPE, $cmd or die "Cannot fork: $!\n";
+        return *PIPE;
+    }
+    my $pid = open PIPE, $mode;
     die "Cannot fork: $!\n" unless defined $pid;
 
     if ($pid) {
-        # Parent process. Return the file handle.
+        # Parent process. Set the encoing layer and return the file handle.
+        binmode PIPE, ":$self->{io_layer}" if PERL58;
         return *PIPE;
     } else {
         # Child process. Execute the commands.
-        exec(@_) or die "Cannot exec $_[0]: $!\n";
+        exec @_ or die "Cannot exec $_[0]: $!\n";
         # Not reached.
     }
 }
@@ -1549,6 +1743,34 @@ sub _read_pipe {
 
 sub _dbpnt { print __PACKAGE__, ": $_[1]\n" }
 
+package SVN::Notify::SMTP;
+
+sub get_handle {
+    my ($class, $notifier) = @_;
+    require Net::SMTP;
+    my $smtp = Net::SMTP->new($notifier->{smtp});
+    binmode tied(*{ $smtp->tied_fh }), ":$notifier->{io_layer}"
+        if SVN::Notify::PERL58;
+    $smtp->mail($notifier->{from});
+    $smtp->to($notifier->{to});
+    $smtp->data;
+    tie local(*SMTP), $class, $smtp;
+    return *SMTP;
+}
+
+sub TIEHANDLE {
+    my ($class, $smtp) = @_;
+    bless \$smtp, $class;
+}
+
+sub PRINT  { ${+shift}->datasend(@_) }
+sub PRINTF { ${+shift}->datasend(sprintf shift, @_) }
+sub CLOSE  {
+    my $self = shift;
+    $$self->dataend;
+    $$self->quit;
+}
+
 1;
 __END__
 
@@ -1558,26 +1780,20 @@ __END__
 
 =item L<SVN::Notify::HTML|SVN::Notify::HTML>
 
+=item L<SVN::Notify::HTML::ColorDiff|SVN::Notify::HTML::ColorDiff>
+
 Subclasses SVN::Notify.
 
+=item L<https://sourceforge.net/docs/E09#svn_notify>
+
+SourceForge.net support for SVN::Notify.
+
 =back
 
 =head1 Bugs
 
 Please send bug reports to <bug-svn-notify@rt.cpan.org>.
 
-=head1 To Do
-
-=over
-
-=item *
-
-Port to Win32. I think it just needs to use Win32::Process to manage
-communication with F<svnlook> and F<sendmail>. See comments in the source
-code.
-
-=back
-
 =head1 Author
 
 =begin comment
@@ -1593,7 +1809,7 @@ David Wheeler <david@kineticode.com>
 
 =head1 Copyright and License
 
-Copyright (c) 2004-2005 Kineticode, Inc. All Rights Reserved.
+Copyright (c) 2004-2006 Kineticode, Inc. All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
